﻿using UnityEngine;
using System.Collections;

public class CongregantBehaviour : MonoBehaviour
{
	// The congregant's taxonomy stats.
	public float age, conservativeness;

	// The congregant's score value, 5 for normal congregants, 10 for rare ones.
	public byte donationvalue;

	// Rare types are either old and liberal, or young and conservative.
	public bool rareType;

	// Happiness value.
	public float happiness = 0;
	public bool leaving;

	// Modifier value for happiness calculations.
	static float happinessModifier = 0.1f;

	ParticleSystem happyParticleSystem, sadParticleSystem;

	void Start()
	{
		happyParticleSystem = transform.FindChild("Happy Particles").GetComponent<ParticleSystem>();
		sadParticleSystem = transform.FindChild("Sad Particles").GetComponent<ParticleSystem>();
	}

	void Update()
	{
		if(happiness<=0 && !leaving)
		{
			StartCoroutine(LeaveChurch());
			leaving = true;
		}
	}

	void OnTriggerEnter2D(Collider2D colliderCheck)
	{
		if(colliderCheck.tag == "Doors")
			Destroy(gameObject);
	}


	/*------------------------------------------------ LEAVE CHURCH ------------------------------------------------*/

	IEnumerator LeaveChurch()
	{
		Vector3 firstPosition, nextPosition;

		// First, step off the pew.
		firstPosition = transform.localPosition + Vector3.right;

		while(transform.localPosition != firstPosition)
		{
			transform.localPosition = Vector2.MoveTowards(transform.localPosition, firstPosition, Time.deltaTime*2);
			yield return null;
		}

		// Second, leave the row.
		nextPosition = transform.parent.FindChild("Aisle Waypoint").transform.position;

		transform.Rotate(0, 0, -90);

		while(transform.position != nextPosition)
		{
			transform.position = Vector2.MoveTowards(transform.position, nextPosition, Time.deltaTime*2);
			yield return null;
		}

		// Lastly, leave the church.
		nextPosition = transform.parent.FindChild("Leave Waypoint").transform.position;

		transform.Rotate(0, 0, -90);

		while(transform.position != nextPosition)
		{
			transform.position = Vector2.MoveTowards(transform.position, nextPosition, Time.deltaTime*2);
			yield return null;
		}
	}


	/*------------------------------------------------ HAPPINESS CALCULATIONS ------------------------------------------------*/

	// Calculate happiness changes based on response. NOTE: in responseValues, x is age and y is conservativeness.
	public void HappinessCalculations(Vector2 responseValues)
	{
		// For a young liberal.
		if(age <= 0 && conservativeness <= 0)
		{
			//Increase happiness.
			if(responseValues.x <= 0 && responseValues.y <= 0)
			{
				print("Happy young liberal");
				happiness += (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				happyParticleSystem.Emit(20);
			}
		
			// Descrease happiness.
			else if(responseValues.x > 0 && responseValues.y > 0)
			{
				print("Sad young liberal");
				happiness -= (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				sadParticleSystem.Emit(20);	
			}
		}

		// For a young conservative.
		else if(age <= 0 && conservativeness > 0)
		{
			//Increase happiness.
			if(responseValues.x <= 0 && responseValues.y > 0)
			{
				print("Happy young conservative");
				happiness += (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				happyParticleSystem.Emit(20);
			}

			// Descrease happiness.
			else if(responseValues.x > 0 && responseValues.y <= 0)
			{
				print("Sad young conservative");
				happiness -= (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				sadParticleSystem.Emit(20);
			}
		}
			
		// For an old liberal.
		else if(age > 0 && conservativeness <= 0)
		{
			//Increase happiness.
			if(responseValues.x > 0 && responseValues.y <= 0)
			{
				print("Happy old liberal");
				happiness += (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				happyParticleSystem.Emit(20);
			}

			// Descrease happiness.
			else if(responseValues.x <= 0 && responseValues.y > 0)
			{
				print("Sad old liberal");
				happiness -= (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				sadParticleSystem.Emit(20);
			}
		}

		// For an old conservative.
		else if(age > 0 && conservativeness > 0)
		{
			//Increase happiness.
			if(responseValues.x > 0 && responseValues.y > 0)
			{
				print("Happy old conservative");
				happiness += (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				happyParticleSystem.Emit(20);
			}

			// Descrease happiness.
			else if(responseValues.x <= 0 && responseValues.y <= 0)
			{
				print("Sad old conservative");
				happiness -= (Mathf.Abs(age) + Mathf.Abs(responseValues.x))*happinessModifier;
				sadParticleSystem.Emit(20);
			}
		}
	}
}
