﻿using UnityEngine;
using UnityEngine.UI;
using System.Collections;
using System.Collections.Generic;


public class CongregationManager : MonoBehaviour
{
	static CongregantBehaviour[,,] congregationArray;
	static CongregantBehaviour[] rareCongregants;

	private ResponseButtons responseButtons;

	private UIManager uiManager;
	Text timerText;

	public float gameTimer = 45;
	private bool gameEnded = false;
	public static bool gameStarted = false;

	public List<Sprite> characterSprites;

	public Color32 brown, blue, green, blonde;

	void Start ()
	{
		// Array containing 6 aisles, of 4 rows, of 4 congregants.
		//
		// The aisles are ordered from top to bottom, left to right:
		// 0      1
		//   2  3
		//   4  5
		//
		// Rare congregants is a sub-set of congregation array, containing any rare type congregants.
		congregationArray = new CongregantBehaviour[6, 4, 4];
		rareCongregants = new CongregantBehaviour[24];

		// Generate rare congregants first, as the GenerateCongregation() will then avoid generating stats for rare congregants afterwards.
		GenerateRareCongregants ();
		GenerateCongregation ();

		responseButtons = GameObject.Find ("Response").GetComponent<ResponseButtons> ();

		uiManager = GameObject.Find ("UI Canvas").GetComponent<UIManager> ();
		timerText = GameObject.Find("Timer Text").GetComponent<Text>();

		responseButtons.Initalize ();
	}

	void Update ()
	{
		if(gameStarted)
		{
			gameTimer -= Time.deltaTime;
			timerText.text = Mathf.RoundToInt(gameTimer).ToString();
			HappinessDecay();
		}

		if (gameTimer <= 0 && !gameEnded)
		{
			gameEnded = true;
			EndGame ();
		}
	}


	/*------------------------------------------------ END GAME ------------------------------------------------*/

	short CalculateScore()
	{
		short totalScore = 0;

		foreach(CongregantBehaviour congregantBehaviour in congregationArray)
		{
			if(congregantBehaviour && !congregantBehaviour.leaving)
				totalScore += congregantBehaviour.donationvalue;
		}

		return totalScore;
	}


	/*------------------------------------------------ END GAME ------------------------------------------------*/

	public void EndGame ()
	{
		gameStarted = false;

		uiManager.ChangeUI ("Score");

		uiManager.ShowScore(CalculateScore());
	}


	/*------------------------------------------------ SEND RESPONSE TO CONGREGANTS ------------------------------------------------*/

	public void SendResponseToCongregants (Vector2 responseValues)
	{
		foreach (CongregantBehaviour congregantBehaviour in congregationArray) {
			if(congregantBehaviour)
				congregantBehaviour.HappinessCalculations (responseValues);
		}
	}


	/*------------------------------------------------ HAPPINESS DECAY ------------------------------------------------*/

	// Tells all congregants to decay their happiness value over time.
	void HappinessDecay ()
	{
		foreach (CongregantBehaviour congregantBehaviour in congregationArray) {
			congregantBehaviour.happiness -= Time.deltaTime;
		}
	}


	/*------------------------------------------------ GENERATE RARE CONGREGANTS ------------------------------------------------*/

	void GenerateRareCongregants ()
	{
		// Get all congregants in the level, and make an array with the indexes of congregants to be made rare.
		GameObject[] congregantObjects = GameObject.FindGameObjectsWithTag ("Congregant");
		byte[] rareCongregantsChosen = new byte[rareCongregants.Length];

		// Cycle through the congregant indexes.
		for (byte counter = 0; counter < rareCongregantsChosen.Length; counter++) {
			// Select an index between 0 and the number of congregant objects randomly, and make sure there are no duplicates.
			bool finished = false;
			while (!finished) {
				rareCongregantsChosen [counter] = (byte)Mathf.RoundToInt (Random.Range (0, congregantObjects.Length));

				foreach (byte rareCongregantCheck in rareCongregantsChosen) {
					if (rareCongregantCheck != rareCongregantsChosen [counter])
						finished = true;
				}	
			}
		}

		// Cycle through the congregant gameObjects.
		for (byte counter = 0; counter < rareCongregantsChosen.Length; counter++) {
			// Cycle through the rare congregants indexes.
			for (byte counter2 = 0; counter2 < congregantObjects.Length; counter2++) {
				// Set the rare congregants behaviours.
				if (counter2 == rareCongregantsChosen [counter])
					rareCongregants [counter] = congregantObjects [counter].GetComponent<CongregantBehaviour> ();
			}
		}

		foreach (CongregantBehaviour rareCongregant in rareCongregants) {
			rareCongregant.rareType = true;

			//Calculate stats of the congregant, making sure they are either old and liberal or young and conservative.
			do {
				rareCongregant.age = Mathf.Round (Random.Range (-50, 50));
				rareCongregant.conservativeness = Mathf.Round (Random.Range (-50, 50));

				rareCongregant.donationvalue = 10;
			} while((rareCongregant.age > 0 && rareCongregant.conservativeness > 0) || (rareCongregant.age < 0 && rareCongregant.conservativeness < 0));
		}
	}


	/*------------------------------------------------ GENERATE CONGREGATION ------------------------------------------------*/

	void GenerateCongregation ()
	{
		// Cycle through the 6 aisles.
		for (byte aisleNumber = 0; aisleNumber < 6; aisleNumber++) {
			// Cycle through the 4 rows in the aisle.
			for (byte rowNumber = 0; rowNumber < 4; rowNumber++) {
				// Cycle through the 4 congregants in the row.
				for (byte congregantNumber = 0; congregantNumber < 4; congregantNumber++) {
					CongregantBehaviour thisCongregant = transform.GetChild (aisleNumber).GetChild (rowNumber).GetChild (congregantNumber).GetComponent<CongregantBehaviour> ();

					if (!thisCongregant.rareType) {
						//Calculate stats of the congregant, making sure they are NOT old and liberal or young and conservative.
						do {
							thisCongregant.age = Mathf.Round (Random.Range (-50, 50));
							thisCongregant.conservativeness = Mathf.Round (Random.Range (-50, 50));

							thisCongregant.donationvalue = 5;
						} while((thisCongregant.age > 0 && thisCongregant.conservativeness < 0) || (thisCongregant.age < 0 && thisCongregant.conservativeness > 0));
					}

					SpriteChange (thisCongregant.gameObject.GetComponent<SpriteRenderer>(), thisCongregant);

					// Add the congregant to the array.
					congregationArray [aisleNumber, rowNumber, congregantNumber] = thisCongregant;
				}
			}
		}
	}

	/*------------------------------------------------ CHANGE SPRITES ------------------------------------------------*/

	void SpriteChange (SpriteRenderer congregantSpriteRenderer, CongregantBehaviour congregantBehaviour)
	{
		int randomNumber = Random.Range(0, characterSprites.Count);
		congregantSpriteRenderer.sprite = characterSprites[randomNumber];

		// Brown: old/lib. Blue: young/lib. Blonde: young/cons. Green: old/cons.
		if(congregantBehaviour.age > 0 && congregantBehaviour.conservativeness < 0)
			congregantSpriteRenderer.color = brown;
		else if(congregantBehaviour.age < 0 && congregantBehaviour.conservativeness < 0)
			congregantSpriteRenderer.color = blue;
		else if(congregantBehaviour.age < 0 && congregantBehaviour.conservativeness > 0)
			congregantSpriteRenderer.color = blonde;
		else
			congregantSpriteRenderer.color = green;
	}
}
