﻿using UnityEngine;
using UnityEngine.UI;

using System.Collections;
using System.Collections.Generic;

public class ResponseButtons : MonoBehaviour
{

	private List< Dictionary<string, Vector2>> allScenarios = new List<Dictionary<string, Vector2>>(1);

	CongregationManager congregationManager;

	/// <summary>
	/// Holds the current response
	/// </summary>

	public Dictionary<string, Vector2> currentScenario{ get; private set; }

	private List<string> responeType = new List<string>(4);
	private List<string> ScenarioName ;


	private Scenarios scenario = new Scenarios();

	public List<Color> typeColours;

	/// <summary>
	/// Initalize the buttons.
	/// </summary>
	public void Initalize()
	{
		SetUpScenarios();
		SetUpButtons();

	}

	void Start()
	{
		congregationManager = GameObject.Find("Congregation").GetComponent<CongregationManager>();
	}

	// ---------------------------- Adds all the scenarios into the list ------------------------- //

	void SetUpScenarios()
	{
		ScenarioName = new List<string>(scenario.allScenarios.Count);

		// adds all the dictionaries in the scenario class
		foreach(string key in scenario.allScenarios.Keys)
		{
			ScenarioName.Add(key);
			allScenarios.Add(scenario.allScenarios[key]);

		}

		StatementAssignments();

	}

	// ---------------------------- Randomly gets a response --------------------------- //

	void StatementAssignments()
	{
		//-- Removes previous scenario
		responeType.Clear(); //-- clears button respose text
		ScenarioName.Remove(GameObject.Find("Scenario").GetComponent<Text>().text);// Removes scenario text
		allScenarios.Remove(currentScenario); // Removes the actual scenario

		int randomScenario = Mathf.RoundToInt(Random.Range(0, allScenarios.Count)); // Gets random number

		GameObject.Find("Scenario").GetComponent<Text>().text = ScenarioName[randomScenario];

		currentScenario = allScenarios[randomScenario];


		AddTextToButtons();
	}


	// ---------------------------- Adds listener to each button to allow it to call Responses ------------------------- //
	void SetUpButtons()
	{
		for(int child = 0; child < transform.childCount; child++)
		{
			Button childButton = transform.GetChild(child).GetComponent<Button>();

			childButton.onClick.AddListener(delegate
			{ 
				Responses(responeType[childButton.transform.GetSiblingIndex()], childButton.transform.GetSiblingIndex());
			});
		}
	}

	// ----------------------------- Called when button clicked ------------------------------- //
	void Responses(string responses, int whichButton)
	{
		congregationManager.SendResponseToCongregants(currentScenario[responses]);

		if(allScenarios.Count > 1)
		{
			StatementAssignments();
		}else{
			congregationManager.EndGame();
		} 
	}


	void AddTextToButtons()
	{
		foreach(string key in currentScenario.Keys)
		{
			responeType.Add(key);
		}

		for(int child = 0; child < transform.childCount; child++)
		{
			if(child < currentScenario.Count)
			{
				if(!transform.GetChild(child).gameObject.activeInHierarchy)
				{
					transform.GetChild(child).gameObject.SetActive(true);
				}

//				Button childButton = transform.GetChild (child).GetComponent<Button>();
				Text childText = transform.GetChild(child).GetChild(0).GetComponent<Text>();
				childText.text = responeType[child];

				if(currentScenario[responeType[child]].x > 0 && currentScenario[responeType[child]].y > 0)
				{
					childText.color = typeColours[0];

				}
				else if(currentScenario[responeType[child]].x > 0 && currentScenario[responeType[child]].y < 0)
				{
					childText.color = typeColours[1];
								
				}
				else if(currentScenario[responeType[child]].x < 0 && currentScenario[responeType[child]].y > 0)
				{
					childText.color = typeColours[2];
										
				}
				else if(currentScenario[responeType[child]].x < 0 && currentScenario[responeType[child]].y < 0)
				{
					childText.color = typeColours[3];
												
				}	
			}
			else
			{
				transform.GetChild(child).gameObject.SetActive(false);
			}
		}
	}
}
