﻿using UnityEngine;
using UnityEngine.SceneManagement;
using UnityEngine.UI;
using System.Collections;

public class StartScreenEvents : MonoBehaviour
{
	ParticleSystem.ShapeModule flamesParticleShape;
	Transform flamesTransform;
	RectTransform criticalRectTransform;

	AudioSource[] audioSources;

	Image criticalImage, massImage;

	void Start()
	{
		flamesTransform = GameObject.Find("Flames").transform;
		flamesParticleShape = flamesTransform.GetComponent<ParticleSystem>().shape;

		criticalRectTransform = GameObject.Find("Critical").GetComponent<RectTransform>();

		audioSources = GetComponents<AudioSource>();

		criticalImage = GameObject.Find("Critical").GetComponent<Image>();
		massImage = GameObject.Find("Mass").GetComponent<Image>();

		StartCoroutine(StartScreenRoutine());
	}

	IEnumerator StartScreenRoutine()
	{
		yield return new WaitForSeconds(1);

		yield return DropCriticalText();
		audioSources[0].PlayDelayed(0);

		yield return new WaitForSeconds(1);

		flamesTransform.GetComponent<ParticleSystem>().Play();
		audioSources[1].PlayDelayed(0);
		yield return StartCoroutine(InitiateFire());
		yield return new WaitForSeconds(1);

		audioSources[2].PlayDelayed(0);
		yield return new WaitForSeconds(3);

		flamesTransform.GetComponent<ParticleSystem>().Stop();

		StartCoroutine(FadeOutImage(massImage));
		yield return StartCoroutine(FadeOutImage(criticalImage));

		SceneManager.LoadScene("Game Scene");
	}

	IEnumerator DropCriticalText()
	{
		Vector2 endPosition = new Vector2(0, 2.5f);
		Vector2 startPosition = criticalRectTransform.anchoredPosition;

		float lerpValue = 0;

		while(lerpValue < 1)
		{
			criticalRectTransform.anchoredPosition = Vector2.Lerp(startPosition, endPosition, lerpValue);

			lerpValue += Time.deltaTime*5;

			yield return null;
		}
	}

	IEnumerator InitiateFire()
	{
		Vector3 endPosition = new Vector3(0, flamesTransform.position.y, 0);
		Vector3 startPosition = flamesTransform.position;

		float lerpValue = 0;

		float placeholderRadius = 0.01f;

		while(lerpValue < 1)
		{
			flamesTransform.position = Vector3.Lerp(startPosition, endPosition, lerpValue);

			placeholderRadius = Mathf.Lerp(0.01f, 10, lerpValue);
			flamesParticleShape.radius = placeholderRadius;

			lerpValue+=Time.deltaTime;

			yield return null;
		}
	}

	IEnumerator FadeOutImage(Image image)
	{
		Color startColor = image.color;
		Color endColor = new Color(1, 1, 1, 0);

		Color placeholderColor = image.color;

		float lerpValue = 0;

		while(lerpValue < 1)
		{
			placeholderColor = Color.Lerp(startColor, endColor, lerpValue);

			image.color = placeholderColor;

			lerpValue += Time.deltaTime*0.75f;

			yield return null;
		}
	}
}
